/* ***** BEGIN LICENSE BLOCK *****
 * Licensed under Version: MPL 1.1/GPL 2.0/LGPL 2.1
 * Full Terms at http://mozile.mozdev.org/license.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Playsophy code.
 *
 * The Initial Developer of the Original Code is Playsophy
 * Portions created by the Initial Developer are Copyright (C) 2002-2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *		Karl Guertin <grayrest@grayrest.com>
 *		James A. Overton <james@overton.ca>
 *		Christian Stocker (Bitflux)
 *		Paul Everitt
 *		David Palm
 *
 * ***** END LICENSE BLOCK ***** */

/**
@fileoverview
Mozile Toolbar.

The code here deals with the active toolbar; removing
and replacing buttons and menus, etc.
*/

function mozileHideToolbar() {mozileTools.toolbar.collapsed=true;}
function mozileShowToolbar() {mozileTools.toolbar.collapsed=false;}

/**
Checks if command is active.

@return "true" or "false"
@type String
*/
function mozileCheckActive(carr) {
	var active="false";
	try {
		var type = carr[6];
		var args = carr[7];
		var e = mozile.lastContextNode;
		if ( e ) {
			var cs = mozileTarget.document.defaultView.getComputedStyle(mozile.lastContextNode, null);

			switch(type) {
				case "setLinesContainer":
					var l = e.__nilParentElement;
					if ( l.nodeNamed( args[1] ) ) { //args[0] is namespace, ignore for now.
						active = "true";
					}
				break;
				
				case "toggleListLines":
					var list = mozileTarget.eDOMXHTML.listContainer(e);
					if ( list && list.nodeNamed( args[0] ) ) {
						active = "true";
					}
				break;
				
				case "toggleStyle":
					active=mozileCheckInlineStyles(e,[args[0]],[args[1]],args[3]);
					if ( active != "true" ) {
						active=mozileCheckStyles(cs,args[0],args[1],args[3]);
					}
				break;

				case "styleText":
					active=mozileCheckInlineStyles(e,[args[0]],[args[1]],1);
					if ( active != "true" ) {
						active=mozileCheckStyles(cs,[args[0]],[args[1]],1);
					}
				break;

				case "styleLines":
					active=mozileCheckInlineStyles(e,[args[0]],[args[1]],1);
					if ( active != "true" ) {
						active=mozileCheckStyles(cs,[args[0]],[args[1]],1);
					}
				break;

				case "toggleAttribute":
					var attrVal=mozile.lastContextNode.getAttribute(args[0]);
					if (args[1]==attrVal) {
						active="true";
					}
				break;
			}
			return active;
		}
	} catch(e) {}

	return active;
}

function mozileCheckInlineStyles(element, name, value, numberActive) {
	var s, v;
	var active="false";
	try {
		if(numberActive>0) {
			for ( s=0; s<numberActive; s++ ) {
				v = element.style.getPropertyValue(name[s]);
				if ( !mozileTarget.eDOM.compareStyleValues(name[s], v, value[s]) ) {
					break;
				}
			}
			if ( s==numberActive ) {
				active="true";
			}
		}
	}catch(e){}
	return active;
}
/*
Only works if style property is absolute.
Relative values are missed by getComputedStyle.

@param computedStyle
@param {Array} name
@param {Array} value
@param {Integer} numberActive
*/
function mozileCheckStyles(computedStyle, name, value, numberActive) {
	var s, v;
	var active="false";
	try {
		if(numberActive>0) {
			for ( s=0; s<numberActive; s++ ) {
				v = computedStyle.getPropertyValue(name[s]);
				if ( !mozileTarget.eDOM.compareStyleValues(name[s], v, value[s]) ) {
					break;
				}
			}
			if ( s==numberActive ) {
				active="true";
			}
		}
	}catch(e){}
	return active;
}


function mozileUpdateDisplay() {
	try{
		mozileUpdateButtons();
		mozileUpdateUndoButtons(true);
		mozileUpdateCommandMenu(true);
		mozileUpdateSource(true);
		mozileUpdateTagsButton(true);
		mozileToolbarSetFixed(true);
	}catch(e){}
}

function mozileClearDisplay() {
	try{
		mozileClearButtons();
		mozileClearUndoButtons();
		mozileClearCommandMenu();
		mozileUpdateSource(true);
		mozileUpdateTagsButton(true);
		mozileToolbarSetFixed(false);
	}catch(e){}
}

/******** Command Menu ********/

function mozileUpdateCommandMenu(silent) {
	var f = ["mozileToolbar.js","mozileUpdateCommandMenu"];
	if(!silent) {mozileDebug(f,3,"Updating Command Menu...");}

	try {
		var contextTag = mozile.lastContextNode.tagName.toLowerCase();
		var tags = mozileTarget.document.mozileMES.tags;
		
		var tagIndex = mozileMES.findByName(contextTag, tags);
		if(tagIndex<0) {
			mozileDebug(f,1,"Context "+ contextTag +" is undefined!");
			mozileClearCommandMenu();	// clears the menu
			return; 
		}
		mozileMakeCommandMenu(tags[tagIndex][3]);
	} catch(e) {
		mozileDebug(f,0,"Exception: "+e);
	}
}

function mozileClearCommandMenu() {
	mozileTools.commandMenu.open=false; //Shouldn't be needed according to the specs but it keeps getting stuck open.
	mozileRemoveChildren(mozileTools.commandMenu);
	mozileTools.commandMenu.setAttribute("hidden", "true");
	return;
}

function mozileMakeCommandMenu(commandsArray) {
	if ( mozileTools.commandMenu.hasAttribute("disabled") && mozileTools.commandMenu.getAttribute("disabled")=="true" ) {
		mozileTools.commandMenu.setAttribute("hidden", "true");
	} else {
		mozileClearCommandMenu();
		if (mozileTarget) {
			var menupop = mozileMakeMenu(commandsArray);
			menupop.setAttribute("id","mozile-command-menu-popup");
			mozileTools.commandMenu.appendChild(menupop);
			mozileTools.commandMenu.setAttribute("hidden", "false");
		}
	}
}

/******** General Menus ********/

function mozileMakeMenu(commandsArray) {
	var menupopup = document.createElement('menupopup');
	for(var i=0; i<commandsArray.length; i++) {
		var menuitem = mozileMakeMenuitem(commandsArray[i]);
		if(menuitem!==0) {
			menupopup.appendChild(menuitem);
		}
	}
	return menupopup;
}


function mozileMakeMenuitem(commandArray) {
		// check to see if this command is in the mesCommands array
	var commands = mozileTarget.document.mozileMES.commands;
	var commandIndex = mozileMES.findByName(commandArray[0], commands);
		// if not, see if it's a commandGroup
	if(commandIndex<0) {
		if (commandArray[0]=="separator") {
			return document.createElement('menuseparator');
		}
		if(commandArray.length!=7) {
			alert("[mozileMakeMenuitem] Can't find command: "+commandArray[0]);
			return 0;
		}	
		
			// if it has the right shape, make a new 
		var garr = new Array();
		garr = commandArray;
		var menu = document.createElement('menu');
		menu.setAttribute('label',garr[1]);
		menu.setAttribute('accesskey',garr[4]);
		menu.setAttribute('tooltiptext',garr[2]);
		var newMenupopup = mozileMakeMenu(garr[6]);		// call this function recursively
		menu.appendChild(newMenupopup);
		return menu;
	}
	
	var carr = new Array();
	carr = commands[commandIndex];
	
	var menuitem = document.createElement('menuitem');
	menuitem.setAttribute('label',carr[1]);
	menuitem.setAttribute('accesskey',commandArray[1]);
	menuitem.setAttribute('tooltiptext',carr[2]);
	menuitem.setAttribute('oncommand','mozileFormat("'+ carr[0] +'")' ); // use the command name
		// if the command is currently active, then put a check next to the menuitem
	if(mozileCheckActive(carr)=="true") {
		menuitem.setAttribute("type","checkbox");
		menuitem.setAttribute("checked","true");
	}
	if ( carr[5] && carr[5]!=="") {
		menuitem.setAttribute('key',carr[5]);
	}

	return menuitem;

}

/******** Toolbar Context Buttons ********/

function mozileClearButtons(silent) {
	var f = ["mozileToolbar.js","mozileClearButtons"];
	if(!silent) {mozileDebug(f,3,"Clearing buttons...");}
	mozileRemoveChildren(mozileTools.toolbar1);
	mozileUpdateToolbar(silent);
}

function mozileUpdateButtons(silent) {
	var f = ["mozileToolbar.js","mozileUpdateButtons"];
	if(!silent) {mozileDebug(f,3,"Updating buttons...");}

	mozileClearButtons(true);
	if (mozileTarget) {
		var mes = mozileTarget.document.mozileMES;
		var contextTag = mozile.lastContextNode.localName.toLowerCase();
		var tagIndex = mozileMES.findByName(contextTag, mes.tags);
		if(tagIndex<0) {
			mozileDebug(f,2,"Context undefined!"); 
		}else{
			mozileMakeButtons(mes.tags[tagIndex][3]);
		}
	}
}

/**
Dim the Mozile fixed toolbar buttons and the main toolbar Edit button.
(Dimmed by setting the opacity via mozile.css)
*/
function mozileToolbarSwitchOff() {
	try {
		var t = mozileTools.toolbar.getElementsByTagName("toolbarbutton");
		for ( var b=0; b<t.length; b++ ) {
			t[b].setAttribute("mozileon", "false");
		}		
		b = document.getElementById("mozile-edit");
		if ( b ) {b.setAttribute("disabled", "true");}		
	} catch (e) {
	}
}

/**
Restore Mozile fixed toolbar buttons to normal.
*/
function mozileToolbarSwitchOn() {
	try {
		var t = mozileTools.toolbar.getElementsByTagName("toolbarbutton");
		for ( var b=0; b<t.length; b++ ) {
			if ( t[b].hasAttribute("mozileon") ) {
				t[b].removeAttribute("mozileon");
			}
		}
		b = document.getElementById("mozile-edit");
		if ( b && b.hasAttribute("disabled") ) {b.removeAttribute("disabled");}		
	} catch (e) {
	}
}

/**
Set fixed toolbar buttons to on or off.
Show/hide Bug button according to "debugLevelInt" preference.
*/
function mozileToolbarSetFixed( value ) {
	value = value ? "true" : "false";
	try {
		mozileTools.buttonSave.setAttribute("mozileon", value);
		mozileTools.buttonCopy.setAttribute("mozileon", value);
		mozileTools.buttonCut.setAttribute("mozileon", value);
		mozileTools.buttonPaste.setAttribute("mozileon", value);
		mozileTools.buttonUndo.setAttribute("mozileon", value);
		mozileTools.buttonRedo.setAttribute("mozileon", value);
		mozileTools.buttonTags.setAttribute("mozileon", value);
		mozileTools.buttonSpecial.setAttribute("mozileon", value);
		if (mozilePrefs.getIntPref("mozile.debugLevelInt") === 0) {
			mozileTools.buttonBug.setAttribute("disabled", "true");
			mozileTools.buttonBug.setAttribute("hidden", "true");
		}
	} catch (e) {
	}
}

/**
If Mozile is ON allow the popup menu to open.
*/
function mozileReloadPopup( menupopup )
{
	if (mozileTools.buttonReload.hasAttribute("mozileon"))
	{
		menupopup.hidePopup();
		return false;
	}
	return true;
}

/**
Disable and hide any of the fixed toolbar items that have been disabled via the MES.
Unhides any hidden buttons if no detection, otherwise updates if MozileTarget
is valid, or if Mozile has been loaded.
Could be split into separate clear and update functions if required. 
*/
function mozileUpdateToolbar(silent) {
	var f = ["mozileToolbar.js","mozileUpdateToolbar"];
	if(!silent) {mozileDebug(f,3,"Updating fixed toolbar buttons...");}
	try {
		var t = mozileTools.toolbar.getElementsByTagName("toolbarbutton");
		var mes = new Array();
	  
		if (mozileTarget) {
			mes = mozileTarget.document.mozileMES.toolbar;
		}else{
			var fWin = document.commandDispatcher.focusedWindow;
			if (mozileIsLoaded(fWin)) {
				mes = fWin.document.mozileMES.toolbar;
			}
		}
		//Set all the buttons to be shown
		for ( var b=0; b<t.length; b++ ) {
			var id = t[b].getAttribute("id");
			var found=false;
			for ( var m=0; m<mes.length; m++ ) {
				if ( id == "mozile-"+mes[m] ) {
					found=true;
					break;
				}
			}
			if ( found ) {			
				t[b].setAttribute("disabled", "true");
				t[b].setAttribute("hidden", "true");
			} else {
				if ( t[b].hasAttribute("disabled") ) {
					t[b].removeAttribute("disabled");
				}
				if ( t[b].hasAttribute("hidden") ) {
					t[b].setAttribute("hidden", "false");
				}
			}
		}
	}catch(e){
		mozileDebug(f,1,"Exception: "+e);
	}
}
/**
Finds the element with the specified id in the Mozile Toolbar.

@param {String} id : should begin with "mozile-..."
@return Element node with matching id, otherwise null
@type Element  
*/
function mozileGetToolbarById( id ) {
	var l = mozileTools.toolbar.getElementsByAttribute("id",id);
	return l.length===0 ? null : l[0];
}

/******** Source Editing ********/
/*
This is a temporary fix until source editing saves state for each editable window.
If the edit-source button has been disabled, and the source window is being displayed,
then switch it off.
*/
function mozileUpdateSource() {
	try {
		var button = document.getElementById("mozile-edit-source");
		if ( button.hasAttribute("active") && button.getAttribute("active")=="true" &&
			   button.hasAttribute("disabled") && button.getAttribute("disabled")=="true" ) {
			mozileShowSourceView(); //Toggle source view off!
		}
	}catch(e){
		var f = ["mozileToolbar.js","mozileUpdateSource"];
		mozileDebug(f,1,"Exception updating Source Edit: "+e);
	}
}
/******** Toolbar Object ********/

var mozileTools = new MozileTools();

function MozileTools() {
	this.imageDir = "chrome://mozile/skin/buttons/";
	this.imageFormat = ".png";
	this.imageOff = "-disabled";
	this.imageUndo1 = this.imageDir+"undo"+this.imageFormat;
	this.imageRedo1 = this.imageDir+"redo"+this.imageFormat;
	this.imageUndo0 = this.imageDir+"undo"+this.imageOff+this.imageFormat;
	this.imageRedo0 = this.imageDir+"redo"+this.imageOff+this.imageFormat;
	this.clearUndoButtons = mozileClearUndoButtons;
	this.updateUndoButtons = mozileUpdateUndoButtons;
	this.clearTagsButton = mozileClearTagsButton;
	this.updateTagsButton = mozileUpdateTagsButton;
	this.show=mozileShowToolbar;
	this.hide=mozileHideToolbar;
	this.init = function() {
		this.toolbar = document.getElementById("mozile-toolbar");
		this.toolbar1 = document.getElementById("mozile-arrowbox");
		this.buttonBug = document.getElementById("mozile-bug");
		this.buttonCopy = document.getElementById("mozile-copy");
		this.buttonCut = document.getElementById("mozile-cut");
		this.buttonPaste = document.getElementById("mozile-paste");
		this.buttonUndo = document.getElementById("mozile-undo");
		this.buttonRedo = document.getElementById("mozile-redo");
		this.buttonReload = document.getElementById("mozile-reload");
		this.buttonSave = document.getElementById("mozile-save");
		this.buttonSpecial = document.getElementById("mozile-special-characters");
		this.buttonTags = document.getElementById("mozile-view-tags");
		this.commandMenu = document.getElementById("mozile-command-menu");
		this.menuKey = document.getElementById("mozile-menu").getAttribute("accesskey").toLowerCase();
		this.commandMenuKey = document.getElementById("mozile-command-menu").getAttribute("accesskey").toLowerCase();
	};
}

/******** Undo Redo Buttons ********/

function mozileClearUndoButtons() {
	mozileTools.buttonUndo.setAttribute( "image", mozileTools.imageUndo0 );
	mozileTools.buttonRedo.setAttribute( "image", mozileTools.imageRedo0 );
}

function mozileUpdateUndoButtons(silent) {
	var f = ["mozileTools.js","mozileUpdateUndoButtons"];
	if(!silent) {mozileDebug(f,3,"Updating Undo Buttons...");}
	try {
		if ( !mozileTarget || !("mozileUndo" in mozileTarget) ) {
			mozileTools.clearUndoButtons();
		} else {
			var mUndo = mozileTarget.mozileUndo;
			mozileTools.buttonUndo.setAttribute("image", mUndo.isUndo ? mozileTools.imageUndo1 : mozileTools.imageUndo0);
			mozileTools.buttonRedo.setAttribute("image", mUndo.isRedo ? mozileTools.imageRedo1 : mozileTools.imageRedo0);
		}
	} catch(e) { 
		mozileTools.clearUndoButtons();
	}
}

function mozileClearTagsButton() {
	mozileTools.buttonTags.setAttribute("active","false");
}

function mozileUpdateTagsButton(silent) {
	var f = ["mozileTools.js","mozileUpdateTagsButton"];
	if(!silent) {mozileDebug(f,3,"Updating Tags Button...");}
	try {
		var target = mozileGetCurrentTarget();
		if (!target) {
			target = document.commandDispatcher.focusedWindow;
			if ( !mozileIsLoaded( target ) ) {
				mozileTools.clearTagsButton();
				return;
			}
		}
		if ( !("mozileTags" in target) ) {
			throw ("Missing .mozileTags");
		}
		var mTags = target.mozileTags;
		mTags.setActive( target, mTags.active );
	} catch(e) { 
		mozileTools.clearTagsButton();
	}
}

/******** General Buttons ********/
function mozileMakeButtons(commandsArray) {
	var button;
	var toolbar = mozileTools.toolbar1;
	for(var i=0; i<commandsArray.length; i++) {
			// only make a button if the button attribute is "true"
			// normal command case
		var commandArray=commandsArray[i];
		if(commandArray.length==3 && commandArray[2]=="true") { 
			button = mozileMakeButton(commandArray);
			if(button!==0) { toolbar.appendChild(button); }
		}
			// command group case
		else if(commandArray.length==7 && commandArray[5]=="true") { 
			button = mozileMakeButton(commandArray);
			if(button!==0) { toolbar.appendChild(button); }
		}
	}
}


function mozileMakeButton(commandArray) {
		// check to see if this command is in the mesCommands array
	var button, image;
	var commandIndex = mozileMES.findByName(commandArray[0], mozileTarget.document.mozileMES.commands);
		// if not, see if it's a commandGroup
	if(commandIndex<0) {
		//If toolbar separator
		if ( commandArray[0] == "separator" ) {
			return document.createElement('toolbarseparator');
		}
	
		if(commandArray.length!=7) {
			alert("[mozileMakeButton] Can't find command: "+commandArray[0]);
			return 0;
		}	
		
			// if it has the right shape, make a menu button
		var garr = new Array();
		garr = commandArray;
		
		if ( garr[3] ) {
			image = mozileTarget.document.mozileMES.images[0]+garr[3]+mozileTarget.document.mozileMES.images[2];
		}
		
		button = document.createElement('toolbarbutton');
		button.id="mozile-"+garr[0]+"-menu";
		button.setAttribute('class','mozile-toolbar-menu');
		// Allow image or label for menu button, but not both.
		if ( image ) {
			button.setAttribute('image',image);
		} else { 
			button.setAttribute('label',garr[1]);
		}
		button.setAttribute('tooltiptext',garr[2]);
		button.setAttribute('type','menu');
		var newMenupopup = mozileMakeMenu(garr[6]);			// call the menu creation function
		button.appendChild(newMenupopup);
		return button;
	}
	
	var carr = new Array();
	carr = mozileTarget.document.mozileMES.commands[commandIndex];

	//Else a normal button.		
	image = mozileTarget.document.mozileMES.images[0]+carr[3]+mozileTarget.document.mozileMES.images[2];
	
	button = document.createElement('toolbarbutton');
	button.id="mozile-"+carr[0];
	button.setAttribute('class','mozile-toolbar-button');
	button.setAttribute('label',carr[1]);
	button.setAttribute('image',image);
	button.setAttribute('tooltiptext',carr[2]);
	button.setAttribute('active',mozileCheckActive(carr));
	button.setAttribute('oncommand','mozileFormat("'+ carr[0] +'")' ); 
	if ( carr[5] && carr[5]!=="") {
		button.setAttribute('key',carr[5]);
	}

	return button;
}
